#ifndef EPUCK_ASEBA_CONTROLLER_H
#define EPUCK_ASEBA_CONTROLLER_H

#include <argos2/common/control_interface/ci_controller.h>
#include <argos2/common/control_interface/e-puck/ci_epuck_wheels_actuator.h>
#include <argos2/common/control_interface/e-puck/ci_epuck_proximity_sensor.h>
#include <argos2/common/control_interface/e-puck/ci_epuck_leds_actuator.h>
#include <argos2/common/control_interface/e-puck/ci_epuck_ground_sensor.h>
#include <argos2/common/control_interface/e-puck/ci_epuck_light_sensor.h>
#include <argos2/common/control_interface/e-puck/ci_epuck_encoder_sensor.h>
#include <argos2/common/control_interface/e-puck/ci_epuck_range_and_bearing_actuator.h>
#include <argos2/common/control_interface/e-puck/ci_epuck_range_and_bearing_sensor.h>
#include <argos2/common/utility/argos_random.h>
#include <argos2/common/utility/math/angles.h>
#include <argos2/common/utility/logging/argos_log.h>

#ifndef ASEBA_ASSERT
#define ASEBA_ASSERT
#endif

#include <aseba/common/types.h>
#include <aseba/common/consts.h>
#include <aseba/vm/vm.h>
#include <aseba/vm/natives.h>
#include <aseba/transport/buffer/vm-buffer.h>
#include <dashel/dashel.h>

#include <vector>
#include <valarray>

using namespace argos;

class CEPuckAsebaController: public CCI_Controller {

private:

   CCI_EPuckWheelsActuator* m_pcWheelsActuator;
   CCI_EPuckProximitySensor* m_pcProximitySensor;
   CCI_EPuckLedsActuator* m_pcLedActuator;
   CCI_EPuckGroundSensor* m_pcGroundSensor;
   CCI_EPuckLightSensor* m_pcLightSensor;
   CCI_EPuckRangeAndBearingActuator* m_pcRangeAndBearingActuator;
   CCI_EPuckRangeAndBearingSensor* m_pcRangeAndBearingSensor;
   CCI_EPuckEncoderSensor* m_pcEncoderSensor;

   CARGoSRandom::CRNG* m_pcRNG;
   CRange<SInt32> m_cRandomRange;

public:

   AsebaVMState m_tVM;
   std::valarray<unsigned short> m_vecBytecode;
   std::valarray<signed short> m_vecStack;

   struct SVariables
   {
      sint16 Id; // internal variables that tell aseba studio how to discriminate among the various e-pucks
      sint16 Source; // ""
      sint16 Args[32]; // ""
      sint16 LSpeed;            // left motor speed
      sint16 RSpeed;            // right motor speed
      sint16 Encoder[2];        // encoder sensors
      sint16 Leds[8];           // body red [0..100] %
      sint16 RABSentMsg;        // message sent by the range and bearing actuator
      sint16 Proximity[8];      // proximity sensors
      sint16 Range[8];          // range part of the range and bearing sensor
      sint16 Bearing[8];        // bearing part of the range and bearing sensor
      sint16 RABReceivedMsg[8]; // messages received by the range and bearing sensor
      sint16 Ground[3];         // ground sensors
      sint16 Light[8];          // light sensors
      sint16 State;             // e-puck controller state
      sint16 Prey;              // 1 = e-puck carrying prey 0 = not carrying
      sint16 Random;            // random number generator
      sint16 User[1024];        // space left for user
   } m_sVariables;

   SInt32 m_nFoodItemIdx;       // -1 no food item, >=0 food item carried
   SInt32 m_nState;

public:

   CEPuckAsebaController();
   virtual ~CEPuckAsebaController() {}

   virtual void Init(TConfigurationNode& t_tree);
   virtual void ControlStep();
   virtual void Reset();
   virtual void Destroy() {}

};

#endif
