#ifndef EPUCK_ASEBA_LOOP_FUNCTION_H
#define EPUCK_ASEBA_LOOP_FUNCTION_H

#include <argos2/simulator/dynamic_linking/loop_functions.h>

#ifndef ASEBA_ASSERT
#define ASEBA_ASSERT
#endif

#include <aseba/common/types.h>
#include <aseba/common/consts.h>

#include <aseba/vm/vm.h>
#include <aseba/vm/natives.h>
#include <aseba/transport/buffer/vm-buffer.h>
#include <dashel/dashel.h>

#include <argos2/simulator/space/entities/epuck_entity.h>
#include "epuck_aseba_controller.h"

/*
 * @file <aseba_loop_functions.h>
 *
 * @brief This file is used to communicate between the simulator and aseba studio
 * every e-puck has its own tab in the studio, making it more confortable to work with
 * a large number of e-pucks.
 * The idea behind this whole thing is to put this loop function between the single e-puck controller and aseba studio:
 * aseba studio think to talk with a single robot with various virtual machines, that is our loop function
 * the loop function takes all the messages a it distribute the right messages to the right e-puck.
 * Note: the idea is taken from "Stephane's playground", /aseba/targets/playground/
 *
 * @author Carlo Pinciroli - <cpinciro@ulb.ac.be>
 * @author Manuele Brambilla - <manu.brambi@gmail.com>
 */

using namespace argos;

class CEPuckAsebaLoopFunction : virtual public CLoopFunctions,
                                virtual public Dashel::Hub {
public:

	 CEPuckAsebaLoopFunction();
	 virtual ~CEPuckAsebaLoopFunction() {}
	 virtual void Init(TConfigurationNode& t_tree);
	 virtual void Reset();
	 virtual void Destroy();
	 virtual void PrePhysicsEngineStep();
	 virtual CColor GetFloorColor(const argos::CVector2& c_position_on_plane);
   inline CSpace& GetSpace() { return m_cSpace; }

protected:

   virtual void connectionCreated(Dashel::Stream* pt_stream);
   virtual void incomingData(Dashel::Stream* pt_stream);
   virtual void connectionClosed(Dashel::Stream* pt_stream, bool b_abnormal);

private:

   CSpace::TAnyEntityMap m_tMapEPucks;
   CEPuckAsebaController* m_pcEPuckAsebaController;
   Dashel::Stream* m_ptStream;
   uint16 m_unLastMsgSource;
   std::valarray<uint8> m_vecLastMsgData;

   bool m_bForaging;
   Real m_fFoodSquareRadius;
   CRange<Real> m_cForagingArenaSideX, m_cForagingArenaSideY;
   std::vector<CVector2> m_cFoodPos;
   CFloorEntity* m_pcFloor;
   CARGoSRandom::CRNG* m_pcRNG;

   std::string m_strOutput;
   std::ofstream m_cOutput;

   UInt32 m_unCollectedFood;
   SInt64 m_nEnergy;
   UInt32 m_unEnergyPerFoodItem;
   UInt32 m_unEnergyPerWalkingRobot;

   bool m_bConstruction;

   friend void AsebaSendBuffer(AsebaVMState* pt_vm, const uint8* pt_data, uint16 un_length);
   friend uint16 AsebaGetBuffer(AsebaVMState* pt_vm, uint8* pt_data, uint16 un_max_length, uint16* pt_source);

};

#endif
