#include "epuck_aseba_qtopengl_user_functions.h"
#include "epuck_aseba_controller.h"

/* CONSTANTS USED FOR DRAWING STUFF */

static const Real BODY_RADIUS = 0.035f;

/****************************************/
/****************************************/

CEPuckAsebaQTOpenGLUserFunctions::CEPuckAsebaQTOpenGLUserFunctions() {
   m_pcDrawStateFont = QFont("Arial", 8);
   m_pcDrawIDFont    = QFont("Arial", 7);
}

/****************************************/
/****************************************/

void CEPuckAsebaQTOpenGLUserFunctions::Draw(CEPuckEntity& c_entity) {
   /* Draw robot ID */
   GetOpenGLWidget().qglColor(QColor(236,163,20));
   GetOpenGLWidget().renderText(-0.1f, -0.1f, 0.4f,
                                QString(c_entity.GetId().c_str()),
                                m_pcDrawIDFont);
   /* Check whether the robot has a food item */
   CEPuckAsebaController& cEPuckController = dynamic_cast<CEPuckAsebaController&>(c_entity.GetControllableEntity().GetController());
   if(cEPuckController.m_nFoodItemIdx >= 0) {
      DrawCircle(0.05f,
                 CVector3(0.0f, 0.0f, 0.2f),
                 CColor::BLUE);
   }
   /* Draw LED colors on the ground */
   CVector2 cVertex(BODY_RADIUS, 0.0f);
   CRadians cAngle(CRadians::TWO_PI / 32);
   const GLfloat fEmissionFactor = 10.0f;
   GLfloat pfColor[]     = { 1.0f, 1.0f, 1.0f, 1.0f };
   const GLfloat pfSpecular[]  = { 0.0f, 0.0f, 0.0f, 1.0f };
   const GLfloat pfShininess[] = { 0.0f                        };
   GLfloat pfEmission[]  = { fEmissionFactor, fEmissionFactor, fEmissionFactor, 1.0f };
   glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR,  pfSpecular);
   glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, pfShininess);
   glPushMatrix();
   for(UInt32 i = 0; i < 8; i++) {
      /* Perform rotation */
      glRotatef(-45.0f, 0.0f, 0.0f, 1.0f);
      /* Set color */
      const CColor& cColor = c_entity.GetLEDEquippedEntity().GetLED(i).GetColor();
      if(cColor != CColor::BLACK) {
         pfColor[0] = cColor.GetRed();
         pfColor[1] = cColor.GetGreen();
         pfColor[2] = cColor.GetBlue();
         pfEmission[0] = cColor.GetRed()   * fEmissionFactor;
         pfEmission[1] = cColor.GetGreen() * fEmissionFactor;
         pfEmission[2] = cColor.GetBlue()  * fEmissionFactor;
         glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, pfColor);
         glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION,  pfEmission);
         /* Draw */
         CVector2 cVertex(BODY_RADIUS * 2, 0.0f);
         glBegin(GL_TRIANGLES);
         glNormal3f(0.0f, 0.0f, 1.0f);      
         for(GLuint i = 0; i <= 4; i++) {         
            glVertex3f(0.0f, 0.0f, 0.01);
            glVertex3f(cVertex.GetX(), cVertex.GetY(), 0.01);
            cVertex.Rotate(cAngle);
            glVertex3f(cVertex.GetX(), cVertex.GetY(), 0.01);
         }
         glEnd();
      }
   }
   glPopMatrix();
}

/****************************************/
/****************************************/

REGISTER_QTOPENGL_USER_FUNCTIONS(CEPuckAsebaQTOpenGLUserFunctions, "epuck_aseba_qtopengl_user_functions")
